@@ -3,7 +3,10 @@ use strict;
 use warnings;
 use 5.010;
 
-our $VERSION = '0.14';
+our $VERSION = '0.16';
+
+our $PANDOC_VERSION;   # a string like '1.16'
+$PANDOC_VERSION ||= $ENV{PANDOC_VERSION};
 
 our %ELEMENTS = (
 
@@ -37,8 +40,8 @@ our %ELEMENTS = (
     LineBreak   => ['Inline'],
     Math        => [ Inline => qw(type content) ],
     RawInline   => [ Inline => qw(format content) ],
-    Link        => [ Inline => qw(content target) ],
-    Image       => [ Inline => qw(content target) ],
+    Link        => [ Inline => qw(attr content target) ],
+    Image       => [ Inline => qw(attr content target) ],
     Note        => [ Inline => 'content' ],
     Span        => [ Inline => qw(attr content) ],
 
@@ -121,10 +124,6 @@ sub Document($$) {
 
 # specific accessors
 
-sub Pandoc::Document::Link::url                   { $_[0]->{c}->[1][0] }
-sub Pandoc::Document::Link::title                 { $_[0]->{c}->[1][1] }
-sub Pandoc::Document::Image::url                  { $_[0]->{c}->[1][0] }
-sub Pandoc::Document::Image::title                { $_[0]->{c}->[1][1] }
 sub Pandoc::Document::DefinitionPair::term        { $_[0]->[0] }
 sub Pandoc::Document::DefinitionPair::definitions { $_[0]->[1] }
 
@@ -159,6 +158,16 @@ sub citation($) {
 }
 
 sub pandoc_json($) {
+    state $ast_to_element = sub { # compile once, use repeatedly, keep private
+        my $class = 'Pandoc::Document::' . $_[0]->{t};
+        if ( 'MetaMap' eq $_[0]->{t} ) {
+            for my $v ( values %{ $_[0]->{c} } ) {
+                $v = $_[1]->($v);
+            }
+        }
+        return $class->new_from_ast($_[0])
+    };
+
     shift if $_[0] =~ /^Pandoc::/;
 
     my $ast = eval { decode_json( $_[0] ) };
@@ -170,15 +179,18 @@ sub pandoc_json($) {
     return unless reftype $ast;
 
     if ( reftype $ast eq 'ARRAY' ) {
-        $ast = Document( $ast->[0]->{unMeta}, $ast->[1] );
+        my $meta = $ast->[0]->{unMeta};
+        for my $v ( values %$meta ) {
+            $v = $ast_to_element->($v, $ast_to_element);
+        }
+        $ast = Document( $meta, $ast->[1] );
     }
-    elsif ( reftype $ast eq 'HASH' ) {
-        $ast = element( $ast->{t}, $ast->{c} );
+    elsif ( reftype $ast eq 'HASH' and $ast->{t} ) {
+        # $ast = element( $ast->{t}, $ast->{c} );
+        $ast = $ast_to_element->($ast, $ast_to_element)
     }
 
-    walk $ast, sub {
-        bless $_[0], 'Pandoc::Document::' . $_[0]->{t};
-    };
+    walk $ast, $ast_to_element;
 
     return $ast;
 }
@@ -222,6 +234,8 @@ sub pandoc_json($) {
     *query     = *Pandoc::Walker::query;
     *transform = *Pandoc::Walker::transform;
 
+    sub new_from_ast { bless $_[1] => $_[0] }
+
     sub string {
 
         # TODO: fix issue #4 to avoid this duplication
@@ -270,6 +284,7 @@ sub pandoc_json($) {
         return 0 unless $self->can('match_attributes');
         return $self->match_attributes($selector);
     }
+
 }
 
 {
@@ -324,6 +339,42 @@ sub pandoc_json($) {
     sub is_meta { 1 }
 }
 
+{
+
+    package Pandoc::Document::LinkageRole;
+    our $VERSION = $PANDOC::Document::VERSION;
+
+    for my $Element ( qw[ Link Image ] ) {
+        no strict 'refs'; #no critic
+        unshift @{"Pandoc::Document::${Element}::ISA"}, __PACKAGE__; # no critic
+    }
+
+    sub url                   { $_[0]->{c}->[-1][0] }
+    sub title                 { $_[0]->{c}->[-1][1] }
+    
+    sub new_from_ast {
+        my ($class, $ast) = @_;
+        if ( 2 == @{ $ast->{c} } ) {
+            # prepend attributes to old-style ast
+            unshift @{ $ast->{c} }, ["",[],[]];
+        }
+        return bless $ast => $class;
+    }
+
+    sub TO_JSON {
+        my ( $self ) = @_;
+        my $ast = {%$self};
+        if (    $Pandoc::Elements::PANDOC_VERSION
+            and ($Pandoc::Elements::PANDOC_VERSION lt '1.16') )
+        {
+            # remove attributes from new-style ast
+            $ast->{c} = [ @{ $ast->{c} }[ 1, 2 ] ];
+        }
+        return $ast;
+    }
+
+}
+
 1;
 __END__
 
@@ -377,8 +428,8 @@ ODT, and ePUB.
 Please make sure to use at least Pandoc 1.12 when processing documents
 
 See also module L<Pandoc::Filter>, command line scripts L<pandocwalk> and
-L<pod2pando>, and the internal modules L<Pandoc::Walker> and
-L<Pandoc::Filter::Lazy>.
+L<pod2pandoc>, and the internal modules L<Pandoc::Walker>,
+L<Pandoc::Filter::Lazy>, and L<Pod::Simple::Pandoc>.
 
 =head2 EXPORTED FUNCTIONS
 
@@ -407,7 +458,8 @@ C<element>.
 =head3 pandoc_json $json
 
 Parse a JSON string, as emitted by pandoc in JSON format. This is the reverse
-to method C<to_json>.
+to method C<to_json> but it can read both old (before Pandoc 1.16) and new
+format.
 
 =head3 attributes { key => $value, ... }
 
@@ -464,6 +516,11 @@ Return the element as JSON encoded string. The following are equivalent:
     $element->to_json;
     JSON->new->utf8->convert_blessed->encode($element);
 
+Note that the JSON format changed from Pandoc 1.15 to Pandoc 1.16 by introduction
+of attributes to L<Link|/Link> and L<Image|/Image> elements. Since Pandoc::Elements
+0.16 the new format is serialized by default. Set the package variable or
+environment variable C<PANDOC_VERSION> to 1.15 or below to use the old format.
+
 =head3 name
 
 Return the name of the element, e.g. "Para" for a L<paragraph element|/Para>.
@@ -473,7 +530,7 @@ Return the name of the element, e.g. "Para" for a L<paragraph element|/Para>.
 Return the element content. For most elements (L<Para|/Para>, L<Emph|/Emph>,
 L<Str|/Str>...) the content is an array reference with child elements. Other
 elements consist of multiple parts; for instance the L<Link|/Link> element has
-a link text (C<content>) and a link target (C<target>) with C<url> and
+attributes (C<attr>) a link text (C<content>) and a link target (C<target>) with C<url> and
 C<title>.
 
 =head3 is_block
@@ -646,9 +703,11 @@ Emphasized text, a list of L<inlines|/INLINE ELEMENTS> (C<content>).
 =head3 Image
 
 Image with alt text (C<content>, a list of L<inlines|/INLINE ELEMENTS>) and
-C<target> (list of C<url> and C<title>).
+C<target> (list of C<url> and C<title>) with attributes (C<attr>).
 
-    Image [ @inlines ], [ $url, $title ]
+    Image attributes { %attr }, [ @inlines ], [ $url, $title ]
+
+Serializing the attributes in JSON can be disabled with C<PANDOC_VERSION>.
 
 =head3 LineBreak
 
@@ -659,9 +718,11 @@ Hard line break
 =head3 Link
 
 Hyperlink with link text (C<content>, a list of L<inlines|/INLINE ELEMENTS>)
-and C<target> (list of C<url> and C<title>).
+and C<target> (list of C<url> and C<title>) with attributes (C<attr>).
 
-    Link [ @inlines ], [ $url, $title ]
+    Link attributes { %attr }, [ @inlines ], [ $url, $title ]
+
+Serializing the attributes in JSON can be disabled with C<PANDOC_VERSION>.
 
 =head3 Math
 
@@ -807,6 +868,10 @@ This module version was last aligned with pandoc-types-1.12.4.
 
 Jakob Voß E<lt>jakob.voss@gbv.deE<gt>
 
+=head1 CONTRIBUTORS
+
+Benct Philip Jonsson E<lt>bpjonsson@gmail.comE<gt>
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2014- Jakob Voß
