@@ -3,7 +3,7 @@ use strict;
 use warnings;
 use 5.010;
 
-our $VERSION = '0.15';
+our $VERSION = '0.16';
 
 use Pod::Simple::SimpleTree;
 use Pandoc::Elements;
@@ -180,7 +180,7 @@ sub _pod_link {
         }
     }
 
-    return Link [ _pod_content( $_[0] ) ], [ $url, '' ];
+    return Link attributes { }, [ _pod_content( $_[0] ) ], [ $url, '' ];
 }
 
 # map data section
